import json
import jsonschema
import pytest
from inventory_provider.routes import poller

DEFAULT_REQUEST_HEADERS = {
    "Content-type": "application/json",
    "Accept": ["application/json"]
}


def test_get_all_interfaces(client):
    rv = client.get(
        '/poller/interfaces',
        headers=DEFAULT_REQUEST_HEADERS)
    assert rv.status_code == 200
    assert rv.is_json
    response_data = json.loads(rv.data.decode('utf-8'))
    jsonschema.validate(response_data, poller.INTERFACE_LIST_SCHEMA)
    response_routers = {ifc['router'] for ifc in response_data}
    assert len(response_routers) > 1, \
        'there should data from be lots of routers'


def test_all_router_interface_speeds(client):
    rv = client.post(
        '/poller/speeds',
        headers=DEFAULT_REQUEST_HEADERS)

    assert rv.status_code == 200
    response = json.loads(rv.data.decode("utf-8"))
    jsonschema.validate(response, poller.INTERFACE_SPEED_LIST_SCHEMA)
    assert response  # at least shouldn't be empty
    response_routers = {ifc['router'] for ifc in response}
    assert len(response_routers) > 1, \
        'there should data from be lots of routers'


def test_eumetsat_multicast(mocker, client):

    # routers don't have snmp acl's for us
    mocker.patch('inventory_provider.juniper.snmp_community_string') \
            .return_value = 'blah'

    rv = client.get(
        '/poller/eumetsat-multicast',
        headers=DEFAULT_REQUEST_HEADERS)
    assert rv.status_code == 200
    assert rv.is_json
    response_data = json.loads(rv.data.decode('utf-8'))
    jsonschema.validate(
        response_data, poller.MULTICAST_SUBSCRIPTION_LIST_SCHEMA)
    assert response_data, "the subscription list shouldn't be empty"


def test_gws_direct(client):

    rv = client.get(
        '/poller/gws/direct',
        headers=DEFAULT_REQUEST_HEADERS)
    assert rv.status_code == 200
    assert rv.is_json
    response_data = json.loads(rv.data.decode('utf-8'))
    jsonschema.validate(response_data, poller.GWS_DIRECT_DATA_SCHEMA)
    assert response_data, "the subscription list shouldn't be empty"


def test_gws_indirect(client):
    rv = client.get(
        '/poller/gws/indirect',
        headers=DEFAULT_REQUEST_HEADERS)
    assert rv.status_code == 200
    assert rv.is_json
    response_data = json.loads(rv.data.decode('utf-8'))
    jsonschema.validate(response_data, poller.SERVICES_LIST_SCHEMA)

    assert response_data  # test data is non-empty
    assert all(s['type'] == 'GWS - INDIRECT' for s in response_data)


def test_all_services_default(client):
    rv = client.get(
        '/poller/services',
        headers=DEFAULT_REQUEST_HEADERS)
    assert rv.status_code == 200
    assert rv.is_json
    response_data = json.loads(rv.data.decode('utf-8'))
    jsonschema.validate(response_data, poller.SERVICES_LIST_SCHEMA)

    assert response_data  # test data is non-empty
    assert all(s['status'] == 'operational' for s in response_data)


def test_all_services_all(client):
    rv = client.get(
        '/poller/services?all=1',
        headers=DEFAULT_REQUEST_HEADERS)
    assert rv.status_code == 200
    assert rv.is_json
    response_data = json.loads(rv.data.decode('utf-8'))
    jsonschema.validate(response_data, poller.SERVICES_LIST_SCHEMA)

    assert response_data  # test data is non-empty
    assert any(s['status'] != 'operational' for s in response_data)


@pytest.mark.parametrize('uri_type,expected_type', [
    ('gws_indirect', 'GWS - INDIRECT'),
    ('gws_upstream', 'GWS - UPSTREAM'),
    ('geant_ip', 'GEANT IP'),
    ('geant_lambda', 'GEANT LAMBDA'),
    # ('gts', 'GTS'),  # these are non-monitored (TODO: make a flag?)
    ('md_vpn_native', 'MD-VPN (NATIVE)'),
    ('md_vpn_proxy', 'MD-VPN (PROXY)'),
    ('cbl1', 'CBL1'),
    ('l3_vpn', 'L3-VPN')
])
def test_all_services_by_type(client, uri_type, expected_type):
    rv = client.get(
        f'/poller/services/{uri_type}',
        headers=DEFAULT_REQUEST_HEADERS)
    assert rv.status_code == 200
    assert rv.is_json
    response_data = json.loads(rv.data.decode('utf-8'))
    jsonschema.validate(response_data, poller.SERVICES_LIST_SCHEMA)

    assert response_data  # test data is non-empty
    assert all(s['type'] == expected_type for s in response_data)


def test_get_service_types(client):
    rv = client.get(
        f'/poller/service-types',
        headers=DEFAULT_REQUEST_HEADERS)
    assert rv.status_code == 200
    assert rv.is_json
    response_data = json.loads(rv.data.decode('utf-8'))
    jsonschema.validate(response_data, poller.STRING_LIST_SCHEMA)

    assert response_data  # test data is non-empty
    # some hard-coded values we expect to see from the test data ...
    assert 'gws_indirect' in response_data
    assert 'geant_ip' in response_data

